import numpy as np
import pyLIMA.fits as pyfit
import pyLIMA.models as pymod

from pyLIMA import event
from pyLIMA import telescopes

np.random.seed(51)


def create_event():
    your_event = event.Event()

    data_1 = np.loadtxt('./examples/data/Survey_1.dat')
    telescope_1 = telescopes.Telescope(name='OGLE',
                                       camera_filter='I',
                                       light_curve=data_1.astype(float),
                                       light_curve_names=['time', 'mag', 'err_mag'],
                                       light_curve_units=['JD', 'mag', 'mag'])

    data_2 = np.loadtxt('./examples/data/Followup_1.dat')
    telescope_2 = telescopes.Telescope(name='LCO',
                                       camera_filter='I',
                                       light_curve=data_2.astype(float),
                                       light_curve_names=['time', 'mag', 'err_mag'],
                                       light_curve_units=['JD', 'mag', 'mag'])

    your_event.telescopes.append(telescope_1)
    your_event.telescopes.append(telescope_2)

    telescope_1.ld_gamma = 0.5
    telescope_2.ld_gamma = 0.5

    return your_event


def test_LM():
    eve = create_event()

    pspl = pymod.FSPLmodel(eve)

    my_fit = pyfit.TRFfit(pspl)
    my_fit.fit()

    values = [my_fit.fit_results[key] for key in my_fit.fit_results.keys()]

    assert np.allclose(values[0], np.array(
        [79.93092166436098, 0.008144359355309872, 10.110765454770114,
         0.022598878807753468, 2917.6397596937854, 208.25164854085628,
         92640.22522809621, 49189.89910261785]))

    assert np.allclose(values[1], 3851.0557824024704)
    assert np.allclose(values[3],
                       np.array([[6.49257550e-07, 1.40782955e-07, -1.97323807e-06,
                                  -3.70762247e-08, 9.85490647e-04, -9.76448768e-04,
                                  1.63793660e-03, 1.71403304e-04],
                                 [1.40782955e-07, 4.64977228e-08, -2.98946979e-06,
                                  -2.69722004e-09, 1.24031034e-03, -1.21483177e-03,
                                  3.43937685e-02, 3.55804762e-03],
                                 [-1.97323807e-06, -2.98946979e-06, 1.41395236e-03,
                                  -2.98528633e-06, -5.13112649e-01, 4.96004753e-01,
                                  -1.38713304e+01, -1.43501424e+00],
                                 [-3.70762247e-08, -2.69722004e-09, -2.98528633e-06,
                                  1.26526588e-08, 1.06192035e-03, -1.02463542e-03,
                                  3.37303951e-02, 3.48908060e-03],
                                 [9.85490647e-04, 1.24031034e-03, -5.13112649e-01,
                                  1.06192035e-03, 1.92433495e+02, -1.86739259e+02,
                                  5.10157189e+03, 5.27775375e+02],
                                 [-9.76448768e-04, -1.21483177e-03, 4.96004753e-01,
                                  -1.02463542e-03, -1.86739259e+02, 1.82314553e+02,
                                  -4.93919738e+03, -5.10979226e+02],
                                 [1.63793660e-03, 3.43937685e-02, -1.38713304e+01,
                                  3.37303951e-02, 5.10157189e+03, -4.93919738e+03,
                                  1.68845252e+05, 1.74649666e+04],
                                 [1.71403304e-04, 3.55804762e-03, -1.43501424e+00,
                                  3.48908060e-03, 5.27775375e+02, -5.10979226e+02,
                                  1.74649666e+04, 1.80653636e+03]]))


def test_TRF():
    eve = create_event()

    pspl = pymod.FSPLmodel(eve)

    my_fit = pyfit.TRFfit(pspl)
    my_fit.fit()

    values = [my_fit.fit_results[key] for key in my_fit.fit_results.keys()]

    assert np.allclose(values[0], np.array(
        [79.93092166436098, 0.008144359355309872, 10.110765454770114,
         0.022598878807753468, 2917.6397596937854, 208.25164854085628,
         92640.22522809621, 49189.89910261785]))

    assert np.allclose(values[1], 3851.0557824024704)

    assert np.allclose(values[3],
                       np.array([[6.49257550e-07, 1.40782955e-07, -1.97323807e-06,
                                  -3.70762247e-08, 9.85490647e-04, -9.76448768e-04,
                                  1.63793660e-03, 1.71403304e-04],
                                 [1.40782955e-07, 4.64977228e-08, -2.98946979e-06,
                                  -2.69722004e-09, 1.24031034e-03, -1.21483177e-03,
                                  3.43937685e-02, 3.55804762e-03],
                                 [-1.97323807e-06, -2.98946979e-06, 1.41395236e-03,
                                  -2.98528633e-06, -5.13112649e-01, 4.96004753e-01,
                                  -1.38713304e+01, -1.43501424e+00],
                                 [-3.70762247e-08, -2.69722004e-09, -2.98528633e-06,
                                  1.26526588e-08, 1.06192035e-03, -1.02463542e-03,
                                  3.37303951e-02, 3.48908060e-03],
                                 [9.85490647e-04, 1.24031034e-03, -5.13112649e-01,
                                  1.06192035e-03, 1.92433495e+02, -1.86739259e+02,
                                  5.10157189e+03, 5.27775375e+02],
                                 [-9.76448768e-04, -1.21483177e-03, 4.96004753e-01,
                                  -1.02463542e-03, -1.86739259e+02, 1.82314553e+02,
                                  -4.93919738e+03, -5.10979226e+02],
                                 [1.63793660e-03, 3.43937685e-02, -1.38713304e+01,
                                  3.37303951e-02, 5.10157189e+03, -4.93919738e+03,
                                  1.68845252e+05, 1.74649666e+04],
                                 [1.71403304e-04, 3.55804762e-03, -1.43501424e+00,
                                  3.48908060e-03, 5.27775375e+02, -5.10979226e+02,
                                  1.74649666e+04, 1.80653636e+03]]))


def test_DE():
    eve = create_event()

    pspl = pymod.FSPLmodel(eve)

    my_fit = pyfit.DEfit(pspl, DE_population_size=1, max_iteration=10,
                         display_progress=False, strategy='best1bin')
    my_fit.fit()

    values = [my_fit.fit_results[key] for key in my_fit.fit_results.keys()]

    assert np.allclose(values[0], np.array(
        [8.19544004e+01, 9.68536408e-02, 9.87770121e+00, 2.47034816e-02]))

    assert np.allclose(values[1], 494271.14139971975)

    assert np.allclose(values[3],
                       np.array([[1.57797803e+02, 4.97559158e-01, 3.44235012e+01,
                                  6.48375707e-03, -4.19371837e+02, 3.91710306e+03,
                                  -2.62657168e+09, 2.73448073e+09, 8.93398205e+05],
                                 [1.33252874e+02, 8.94243644e-03, 3.41056594e+02,
                                  2.22096438e-02, -4.68563265e+00, 3.45331338e+03,
                                  -3.27745038e+07, 2.17039019e+08, 9.00712040e+05],
                                 [2.55060755e+01, 3.89867561e-01, 4.44552142e+02,
                                  4.06121453e-02, 6.31520038e+02, 1.79810798e+03,
                                  1.04951197e+09, -2.72407440e+09, 8.98046567e+05],
                                 [9.49287107e+01, 8.58440352e-01, 1.15843863e+02,
                                  3.04967806e-02, 4.17164082e+03, -2.33548147e+03,
                                  -3.25737712e+09, 4.74503126e+09, 8.61381197e+05],
                                 [4.75732686e+01, 6.49966282e-01, 2.17490852e+02,
                                  1.39158499e-02, 1.66513637e+03, 6.16245690e+02,
                                  2.00559077e+09, -3.48260620e+09, 8.89525809e+05],
                                 [9.49287107e+01, 8.58440352e-01, 2.77534867e+02,
                                  3.04967806e-02, 1.48733811e+04, -1.80877870e+04,
                                  -1.81729838e+10, 2.65982124e+10, 8.68050881e+05],
                                 [1.33252874e+02, 8.94243644e-03, 1.69443439e+02,
                                  2.17534734e-02, -5.19969191e+00, 3.42551795e+03,
                                  -6.78548075e+07, 2.28934581e+08, 9.02511318e+05],
                                 [3.68061011e+01, 9.38806305e-01, 2.12377292e+02,
                                  3.44158049e-02, 3.35400321e+03, -1.12997952e+03,
                                  5.02615220e+09, -6.88852089e+09, 8.94487262e+05],
                                 [1.57797803e+02, 8.16062500e-01, 6.49407245e+01,
                                  2.84302359e-02, -7.22967854e+02, 4.27286306e+03,
                                  -1.23384148e+09, 1.41845406e+09, 8.97801441e+05],
                                 [4.75732686e+01, 7.42120606e-02, 1.26734882e+02,
                                  3.48666790e-02, 1.73041513e+00, 3.36412324e+03,
                                  4.05981832e+07, -1.51506442e+08, 9.05460030e+05],
                                 [8.19544004e+01, 8.58440352e-01, 1.15843863e+02,
                                  3.04967806e-02, 4.77251323e+03, -3.14673376e+03,
                                  -3.88249923e+10, 5.68816912e+10, 8.37060848e+05],
                                 [1.00128072e+02, 8.94243644e-03, 6.21138411e+01,
                                  3.92613666e-02, 1.99704545e+00, 3.36237667e+03,
                                  -2.48865263e+07, 8.48730625e+07, 9.08089678e+05],
                                 [3.68061011e+01, 5.19598777e-01, 1.81160015e+02,
                                  3.48808331e-02, 5.21952684e+02, 2.42472282e+03,
                                  6.40003198e+08, -1.24768113e+09, 8.97260113e+05],
                                 [6.37807288e+01, 7.48242139e-01, 1.69573886e+02,
                                  6.48375707e-03, 3.13339504e+03, -1.35064508e+03,
                                  3.57806129e+09, -5.69581928e+09, 8.74317521e+05],
                                 [7.23479005e+01, 6.49966282e-01, 2.17490852e+02,
                                  1.39158499e-02, 3.82466146e+03, -3.06135148e+03,
                                  7.20279265e+09, -1.27471559e+10, 8.67096894e+05],
                                 [7.58953216e+01, 9.27945339e-01, 8.19548880e+01,
                                  3.04967806e-02, 4.25085072e+03, -2.03265445e+03,
                                  5.86863257e+09, -8.19618709e+09, 8.45593764e+05],
                                 [1.01032038e+02, 7.23665033e-01, 3.41056594e+02,
                                  2.22096438e-02, 9.96718605e+03, -1.27028100e+04,
                                  -1.07572398e+10, 1.76005976e+10, 8.77793327e+05],
                                 [9.54469405e+01, 9.38806305e-01, 2.75124163e+02,
                                  2.46357547e-02, 1.93978741e+04, -2.32524231e+04,
                                  -2.35634089e+10, 3.27078005e+10, 8.69041251e+05],
                                 [9.54469405e+01, 7.63121347e-01, 2.75124163e+02,
                                  2.46357547e-02, 9.93683503e+03, -1.20069267e+04,
                                  -1.15385528e+10, 1.82143151e+10, 8.68403702e+05],
                                 [8.59044256e+01, 6.49966282e-01, 1.62474164e+02,
                                  2.87809287e-02, 3.37704442e+03, -2.16289987e+03,
                                  -6.77796218e+09, 1.20065355e+10, 8.49281831e+05],
                                 [8.19544004e+01, 7.09657792e-01, 1.15843863e+02,
                                  3.04967806e-02, 3.03845808e+03, -1.16135411e+03,
                                  -2.03326404e+10, 3.38075985e+10, 8.31871327e+05],
                                 [7.05279546e+01, 5.15598480e-01, 5.76596269e+01,
                                  2.22096438e-02, 1.43589983e+03, 1.31708312e+03,
                                  2.35573769e+08, -4.76800574e+08, 8.05589814e+05],
                                 [9.54469405e+01, 4.01806236e-01, 2.75124163e+02,
                                  1.66446108e-02, 1.68979854e+03, -7.10012592e+02,
                                  -1.48505340e+09, 3.88851607e+09, 8.64456949e+05],
                                 [5.91015087e+01, 7.63121347e-01, 2.75124163e+02,
                                  3.39406497e-02, 5.29286018e+03, -4.77025663e+03,
                                  7.80009670e+09, -1.22871063e+10, 8.82628887e+05],
                                 [7.05279546e+01, 2.09611060e-01, 1.62474164e+02,
                                  1.54421728e-02, 4.38935436e+02, 2.05339530e+03,
                                  1.10334831e+08, -5.11353802e+08, 8.21573958e+05],
                                 [5.25991179e+01, 4.44556309e-01, 5.76596269e+01,
                                  2.22096438e-02, 3.35948742e+02, 2.86424158e+03,
                                  1.61194088e+08, -2.80290495e+08, 8.94114756e+05],
                                 [3.74156996e+01, 6.66805185e-01, 1.15843863e+02,
                                  2.96044388e-02, 5.80133959e+02, 2.51384454e+03,
                                  6.86583345e+08, -1.08205140e+09, 8.98863667e+05],
                                 [5.53445363e+01, 4.01806236e-01, 2.75124163e+02,
                                  1.66446108e-02, 8.97652361e+02, 1.25258211e+03,
                                  8.95277206e+08, -2.30588124e+09, 8.82686903e+05],
                                 [3.74156996e+01, 7.63121347e-01, 2.75124163e+02,
                                  2.06118488e-02, 2.69855217e+03, -7.35964682e+02,
                                  4.12313910e+09, -6.42567907e+09, 8.94164602e+05],
                                 [8.57113729e+01, 7.73463766e-01, 1.62474164e+02,
                                  3.32215600e-02, 5.31356601e+03, -4.52756874e+03,
                                  -1.25742494e+10, 1.96940867e+10, 8.51946809e+05],
                                 [1.04423176e+02, 5.15598480e-01, 5.76596269e+01,
                                  3.47149078e-02, 5.40121225e+02, 2.57570505e+03,
                                  -1.89494670e+08, 3.34574680e+08, 8.88081888e+05],
                                 [3.66327328e+01, 2.54171384e-01, 1.15843863e+02,
                                  2.05740676e-02, 1.92816697e+01, 3.32621806e+03,
                                  1.27898205e+08, -2.98806884e+08, 9.05769653e+05],
                                 [3.66327328e+01, 1.78913733e-01, 2.75124163e+02,
                                  1.66446108e-02, 6.43611408e+01, 3.12438132e+03,
                                  9.83359783e+07, -4.16229684e+08, 9.02532305e+05],
                                 [9.54469405e+01, 9.68536408e-02, 2.74315357e+02,
                                  2.46357547e-02, 1.42052466e+02, 2.56129746e+03,
                                  -2.78230131e+07, 2.56055102e+08, 8.49422502e+05],
                                 [7.05279546e+01, 9.68536408e-02, 1.62474164e+02,
                                  1.54421728e-02, 2.20184805e+02, 2.48850773e+03,
                                  1.60744298e+07, -1.38058239e+08, 7.82815150e+05],
                                 [8.61182319e+01, 3.61628756e-01, 2.41860913e+02,
                                  2.67491674e-02, 1.34094955e+03, -3.74269130e+01,
                                  -2.24215057e+09, 6.49512925e+09, 8.46282466e+05],
                                 [8.19544004e+01, 9.68536408e-02, 9.87770121e+00,
                                  3.04967806e-02, 1.65352619e+03, 1.55879340e+03,
                                  -1.09478997e+06, 9.22407948e+06, 4.94763404e+05],
                                 [8.19544004e+01, 6.68190369e-01, 4.29006852e+02,
                                  3.97303385e-02, 1.52941097e+04, -2.28436734e+04,
                                  -2.28107979e+11, 3.96228604e+11, 8.51367987e+05],
                                 [9.54469405e+01, 8.76387206e-01, 3.21642620e+02,
                                  4.30950750e-02, 2.04029270e+04, -2.58265947e+04,
                                  -2.52468846e+10, 3.65007270e+10, 8.69303132e+05],
                                 [9.75446778e+01, 9.68536408e-02, 1.55458089e+02,
                                  2.22096438e-02, 1.03432967e+02, 2.93730507e+03,
                                  -1.76973270e+07, 1.25361220e+08, 8.71395851e+05],
                                 [6.81540858e+01, 6.81222965e-01, 1.68097819e+02,
                                  4.59356825e-02, 2.84205473e+03, -1.14981038e+03,
                                  3.41233087e+09, -5.81722079e+09, 8.67754956e+05],
                                 [8.19544004e+01, 6.81222965e-01, 1.62474164e+02,
                                  1.54421728e-02, 3.74344069e+03, -2.59430349e+03,
                                  -3.49267804e+10, 5.97966664e+10, 8.40604398e+05],
                                 [7.02673092e+01, 5.25150347e-01, 5.87495665e+01,
                                  3.97303385e-02, 1.44679144e+03, 1.30108245e+03,
                                  2.51635887e+08, -5.02630705e+08, 8.09235452e+05],
                                 [9.39080824e+01, 9.68536408e-02, 2.74315357e+02,
                                  2.46357547e-02, 1.54937763e+02, 2.49086016e+03,
                                  -2.89428729e+07, 2.72659074e+08, 8.40260278e+05],
                                 [7.05279546e+01, 9.68536408e-02, 1.24270071e+02,
                                  2.22096438e-02, 2.34572602e+02, 2.57836433e+03,
                                  1.25365423e+07, -9.75026330e+07, 7.74509420e+05],
                                 [1.09918485e+02, 9.68536408e-02, 9.87770121e+00,
                                  4.14928753e-02, -3.74133701e+01, 3.42135889e+03,
                                  -1.94260991e+09, 1.97941394e+09, 9.07692725e+05],
                                 [5.36785681e+01, 6.09123089e-01, 1.62474164e+02,
                                  4.85508398e-02, 1.28432989e+03, 1.23038383e+03,
                                  1.13944608e+09, -2.06297202e+09, 8.83645920e+05],
                                 [8.19544004e+01, 9.68536408e-02, 5.55721681e+01,
                                  3.97303385e-02, 5.76696501e+02, 2.22462522e+03,
                                  -1.02823583e+07, 1.07993791e+08, 5.22372199e+05],
                                 [9.39080824e+01, 9.68536408e-02, 6.45311203e+01,
                                  4.15406890e-02, 1.38358165e+02, 3.03402673e+03,
                                  -1.46091899e+07, 6.83064189e+07, 8.66895906e+05],
                                 [7.05279546e+01, 9.68536408e-02, 2.71508134e+02,
                                  2.22096438e-02, 1.97042426e+02, 2.29447192e+03,
                                  3.21417467e+07, -3.10082542e+08, 8.08506597e+05],
                                 [8.19544004e+01, 9.68536408e-02, 9.87770121e+00,
                                  2.47034816e-02, 1.65875945e+03, 1.55324878e+03,
                                  -1.10156489e+06, 9.25794813e+06, 4.94271141e+05],
                                 [9.17360186e+01, 9.68536408e-02, 1.62474164e+02,
                                  1.54421728e-02, 1.74480518e+02, 2.64309562e+03,
                                  -1.60289724e+07, 1.38770699e+08, 8.19943282e+05],
                                 [7.17214413e+01, 9.68536408e-02, 4.88437852e+02,
                                  3.97303385e-02, 2.00736201e+02, 1.89526941e+03,
                                  9.57747285e+07, -9.91674859e+08, 8.35476166e+05],
                                 [8.19544004e+01, 9.68536408e-02, 2.74315357e+02,
                                  2.46357547e-02, 2.33644365e+02, 2.07470549e+03,
                                  -2.39020158e+08, 2.49437755e+09, 7.66114981e+05],
                                 [8.19544004e+01, 9.68536408e-02, 2.36249770e+02,
                                  2.22096438e-02, 2.44503195e+02, 2.12790770e+03,
                                  -1.78290502e+08, 1.85783363e+09, 7.50415516e+05]]))


def test_MCMC():
    eve = create_event()

    pspl = pymod.FSPLmodel(eve)

    my_fit = pyfit.MCMCfit(pspl, MCMC_walkers=2, MCMC_links=10, )

    my_fit.model_parameters_guess = [79.93092166436098, 0.008144359355309872,
                                     10.110765454770114,
                                     0.022598878807753468, ]
    my_fit.fit()

    values = [my_fit.fit_results[key] for key in my_fit.fit_results.keys()]

    assert np.allclose(values[0], np.array(
        [7.99310955e+01, 8.15180037e-03, 1.01109318e+01, 2.25348532e-02,
         2.91742259e+03, 2.08481976e+02, 9.24740698e+04, 5.06397500e+04]))

    assert np.allclose(values[1],-20417.332849174007)

    assert np.allclose(values[2],
                       np.array([[[7.99340412e+01, 8.93351065e-03, 1.01027207e+01,
                                   2.13395926e-02, -2.05851177e+04],
                                  [7.99351442e+01, 8.16649329e-03, 1.01115540e+01,
                                   2.16262514e-02, -2.05134881e+04],
                                  [7.99269286e+01, 8.26155566e-03, 1.01110993e+01,
                                   2.23400780e-02, -2.05120101e+04],
                                  [7.99345918e+01, 7.82554947e-03, 1.01156067e+01,
                                   2.27701567e-02, -2.05063843e+04],
                                  [7.99291265e+01, 8.20769719e-03, 1.01098599e+01,
                                   2.25719736e-02, -2.04311251e+04],
                                  [7.99309311e+01, 8.16057122e-03, 1.01055938e+01,
                                   2.43439217e-02, -2.07788349e+04],
                                  [7.99329478e+01, 8.54430170e-03, 1.01032115e+01,
                                   2.32435889e-02, -2.04871966e+04],
                                  [7.99344734e+01, 8.48557085e-03, 1.01053300e+01,
                                   2.29537913e-02, -2.04762004e+04]],

                                 [[7.99341021e+01, 8.81100454e-03, 1.01041455e+01,
                                   2.14977686e-02, -2.05383543e+04],
                                  [7.99351442e+01, 8.16649329e-03, 1.01115540e+01,
                                   2.16262514e-02, -2.05134881e+04],
                                  [7.99290309e+01, 8.36030627e-03, 1.01083444e+01,
                                   2.26556341e-02, -2.04359112e+04],
                                  [7.99345918e+01, 7.82554947e-03, 1.01156067e+01,
                                   2.27701567e-02, -2.05063843e+04],
                                  [7.99291265e+01, 8.20769719e-03, 1.01098599e+01,
                                   2.25719736e-02, -2.04311251e+04],
                                  [7.99309311e+01, 8.16057122e-03, 1.01055938e+01,
                                   2.43439217e-02, -2.07788349e+04],
                                  [7.99335199e+01, 8.52227957e-03, 1.01040059e+01,
                                   2.31349243e-02, -2.04805003e+04],
                                  [7.99344734e+01, 8.48557085e-03, 1.01053300e+01,
                                   2.29537913e-02, -2.04762004e+04]],

                                 [[7.99316033e+01, 8.58892543e-03, 1.01062145e+01,
                                   2.20683005e-02, -2.04579997e+04],
                                  [7.99348754e+01, 8.00058291e-03, 1.01135261e+01,
                                   2.21828996e-02, -2.04686810e+04],
                                  [7.99289507e+01, 8.35318087e-03, 1.01084108e+01,
                                   2.26739396e-02, -2.04362580e+04],
                                  [7.99345918e+01, 7.82554947e-03, 1.01156067e+01,
                                   2.27701567e-02, -2.05063843e+04],
                                  [7.99291265e+01, 8.20769719e-03, 1.01098599e+01,
                                   2.25719736e-02, -2.04311251e+04],
                                  [7.99309311e+01, 8.16057122e-03, 1.01055938e+01,
                                   2.43439217e-02, -2.07788349e+04],
                                  [7.99344188e+01, 8.64788798e-03, 1.01034544e+01,
                                   2.27150827e-02, -2.04415480e+04],
                                  [7.99344734e+01, 8.48557085e-03, 1.01053300e+01,
                                   2.29537913e-02, -2.04762004e+04]],

                                 [[7.99316033e+01, 8.58892543e-03, 1.01062145e+01,
                                   2.20683005e-02, -2.04579997e+04],
                                  [7.99348754e+01, 8.00058291e-03, 1.01135261e+01,
                                   2.21828996e-02, -2.04686810e+04],
                                  [7.99289507e+01, 8.35318087e-03, 1.01084108e+01,
                                   2.26739396e-02, -2.04362580e+04],
                                  [7.99345918e+01, 7.82554947e-03, 1.01156067e+01,
                                   2.27701567e-02, -2.05063843e+04],
                                  [7.99291898e+01, 8.20604559e-03, 1.01097104e+01,
                                   2.26340742e-02, -2.04280112e+04],
                                  [7.99309311e+01, 8.16057122e-03, 1.01055938e+01,
                                   2.43439217e-02, -2.07788349e+04],
                                  [7.99344089e+01, 8.69518650e-03, 1.01027555e+01,
                                   2.27119150e-02, -2.04396332e+04],
                                  [7.99344962e+01, 8.48766653e-03, 1.01053283e+01,
                                   2.29448274e-02, -2.04753505e+04]],

                                 [[7.99323332e+01, 8.61657145e-03, 1.01053146e+01,
                                   2.22357502e-02, -2.04325453e+04],
                                  [7.99348754e+01, 8.00058291e-03, 1.01135261e+01,
                                   2.21828996e-02, -2.04686810e+04],
                                  [7.99315178e+01, 8.11307054e-03, 1.01116855e+01,
                                   2.27177253e-02, -2.04216750e+04],
                                  [7.99345918e+01, 7.82554947e-03, 1.01156067e+01,
                                   2.27701567e-02, -2.05063843e+04],
                                  [7.99292032e+01, 8.20551134e-03, 1.01097217e+01,
                                   2.26345548e-02, -2.04278187e+04],
                                  [7.99309586e+01, 8.16479657e-03, 1.01055714e+01,
                                   2.43310231e-02, -2.07745043e+04],
                                  [7.99344089e+01, 8.69518650e-03, 1.01027555e+01,
                                   2.27119150e-02, -2.04396332e+04],
                                  [7.99344962e+01, 8.48766653e-03, 1.01053283e+01,
                                   2.29448274e-02, -2.04753505e+04]],

                                 [[7.99332613e+01, 8.65172140e-03, 1.01041704e+01,
                                   2.24486505e-02, -2.04213216e+04],
                                  [7.99348754e+01, 8.00058291e-03, 1.01135261e+01,
                                   2.21828996e-02, -2.04686810e+04],
                                  [7.99305118e+01, 8.20716901e-03, 1.01104021e+01,
                                   2.27005658e-02, -2.04183565e+04],
                                  [7.99341984e+01, 7.86234467e-03, 1.01151049e+01,
                                   2.27634468e-02, -2.04887631e+04],
                                  [7.99313450e+01, 8.12812854e-03, 1.01111583e+01,
                                   2.24640058e-02, -2.04186836e+04],
                                  [7.99309586e+01, 8.16479657e-03, 1.01055714e+01,
                                   2.43310231e-02, -2.07745043e+04],
                                  [7.99344089e+01, 8.69518650e-03, 1.01027555e+01,
                                   2.27119150e-02, -2.04396332e+04],
                                  [7.99344962e+01, 8.48766653e-03, 1.01053283e+01,
                                   2.29448274e-02, -2.04753505e+04]],

                                 [[7.99332613e+01, 8.65172140e-03, 1.01041704e+01,
                                   2.24486505e-02, -2.04213216e+04],
                                  [7.99347258e+01, 8.14778774e-03, 1.01112378e+01,
                                   2.22977209e-02, -2.04548938e+04],
                                  [7.99305118e+01, 8.20716901e-03, 1.01104021e+01,
                                   2.27005658e-02, -2.04183565e+04],
                                  [7.99341997e+01, 7.98034340e-03, 1.01133551e+01,
                                   2.27533857e-02, -2.04778936e+04],
                                  [7.99310955e+01, 8.15180037e-03, 1.01109318e+01,
                                   2.25348532e-02, -2.04173328e+04],
                                  [7.99309586e+01, 8.16479657e-03, 1.01055714e+01,
                                   2.43310231e-02, -2.07745043e+04],
                                  [7.99342074e+01, 8.65790083e-03, 1.01033080e+01,
                                   2.26956142e-02, -2.04366187e+04],
                                  [7.99344962e+01, 8.48766653e-03, 1.01053283e+01,
                                   2.29448274e-02, -2.04753505e+04]],

                                 [[7.99332613e+01, 8.65172140e-03, 1.01041704e+01,
                                   2.24486505e-02, -2.04213216e+04],
                                  [7.99347258e+01, 8.14778774e-03, 1.01112378e+01,
                                   2.22977209e-02, -2.04548938e+04],
                                  [7.99308445e+01, 8.18950154e-03, 1.01106330e+01,
                                   2.26988489e-02, -2.04180149e+04],
                                  [7.99342832e+01, 8.00690492e-03, 1.01130193e+01,
                                   2.26811041e-02, -2.04708999e+04],
                                  [7.99310955e+01, 8.15180037e-03, 1.01109318e+01,
                                   2.25348532e-02, -2.04173328e+04],
                                  [7.99309586e+01, 8.16479657e-03, 1.01055714e+01,
                                   2.43310231e-02, -2.07745043e+04],
                                  [7.99342074e+01, 8.65790083e-03, 1.01033080e+01,
                                   2.26956142e-02, -2.04366187e+04],
                                  [7.99339916e+01, 8.55470116e-03, 1.01048551e+01,
                                   2.27420840e-02, -2.04399044e+04]],

                                 [[7.99332613e+01, 8.65172140e-03, 1.01041704e+01,
                                   2.24486505e-02, -2.04213216e+04],
                                  [7.99347258e+01, 8.14778774e-03, 1.01112378e+01,
                                   2.22977209e-02, -2.04548938e+04],
                                  [7.99308445e+01, 8.18950154e-03, 1.01106330e+01,
                                   2.26988489e-02, -2.04180149e+04],
                                  [7.99342832e+01, 8.00690492e-03, 1.01130193e+01,
                                   2.26811041e-02, -2.04708999e+04],
                                  [7.99315707e+01, 8.26149028e-03, 1.01094483e+01,
                                   2.25159390e-02, -2.04176037e+04],
                                  [7.99321101e+01, 8.33957421e-03, 1.01047691e+01,
                                   2.37513629e-02, -2.06049899e+04],
                                  [7.99342074e+01, 8.65790083e-03, 1.01033080e+01,
                                   2.26956142e-02, -2.04366187e+04],
                                  [7.99340380e+01, 8.57689478e-03, 1.01045224e+01,
                                   2.27320904e-02, -2.04391352e+04]],

                                 [[7.99332613e+01, 8.65172140e-03, 1.01041704e+01,
                                   2.24486505e-02, -2.04213216e+04],
                                  [7.99333819e+01, 8.16223130e-03, 1.01110284e+01,
                                   2.24366129e-02, -2.04322697e+04],
                                  [7.99308445e+01, 8.18950154e-03, 1.01106330e+01,
                                   2.26988489e-02, -2.04180149e+04],
                                  [7.99342832e+01, 8.00690492e-03, 1.01130193e+01,
                                   2.26811041e-02, -2.04708999e+04],
                                  [7.99315707e+01, 8.26149028e-03, 1.01094483e+01,
                                   2.25159390e-02, -2.04176037e+04],
                                  [7.99325973e+01, 8.47168213e-03, 1.01045157e+01,
                                   2.32000249e-02, -2.04752432e+04],
                                  [7.99342074e+01, 8.65790083e-03, 1.01033080e+01,
                                   2.26956142e-02, -2.04366187e+04],
                                  [7.99328393e+01, 8.42366212e-03, 1.01069155e+01,
                                   2.26270778e-02, -2.04230714e+04]]]))

    assert np.allclose(values[3],
                       np.array([[[7.99340412e+01, 8.93351065e-03, 1.01027207e+01,
                                   2.13395926e-02, 2.92395385e+03, 2.01832312e+02,
                                   9.07785861e+04, 6.64835672e+04, -2.05851177e+04],
                                  [7.99351442e+01, 8.16649329e-03, 1.01115540e+01,
                                   2.16262514e-02, 2.91158801e+03, 2.14796231e+02,
                                   8.96790842e+04, 7.40985184e+04, -2.05134881e+04],
                                  [7.99269286e+01, 8.26155566e-03, 1.01110993e+01,
                                   2.23400780e-02, 2.92437780e+03, 2.00961587e+02,
                                   9.30283206e+04, 4.93396532e+04, -2.05120101e+04],
                                  [7.99345918e+01, 7.82554947e-03, 1.01156067e+01,
                                   2.27701567e-02, 2.90760700e+03, 2.18884079e+02,
                                   9.17515503e+04, 5.35097619e+04, -2.05063843e+04],
                                  [7.99291265e+01, 8.20769719e-03, 1.01098599e+01,
                                   2.25719736e-02, 2.92108333e+03, 2.04567829e+02,
                                   9.30371060e+04, 4.71959588e+04, -2.04311251e+04],
                                  [7.99309311e+01, 8.16057122e-03, 1.01055938e+01,
                                   2.43439217e-02, 2.91825246e+03, 2.07749586e+02,
                                   9.64448932e+04, 1.26819672e+04, -2.07788349e+04],
                                  [7.99329478e+01, 8.54430170e-03, 1.01032115e+01,
                                   2.32435889e-02, 2.92047010e+03, 2.05502707e+02,
                                   9.43521783e+04, 3.18721533e+04, -2.04871966e+04],
                                  [7.99344734e+01, 8.48557085e-03, 1.01053300e+01,
                                   2.29537913e-02, 2.91715878e+03, 2.09002328e+02,
                                   9.33281586e+04, 4.01340309e+04, -2.04762004e+04]],

                                 [[7.99341021e+01, 8.81100454e-03, 1.01041455e+01,
                                   2.14977686e-02, 2.92206275e+03, 2.03807708e+02,
                                   9.08598537e+04, 6.52759111e+04, -2.05383543e+04],
                                  [7.99351442e+01, 8.16649329e-03, 1.01115540e+01,
                                   2.16262514e-02, 2.91158801e+03, 2.14796231e+02,
                                   8.96790842e+04, 7.40985184e+04, -2.05134881e+04],
                                  [7.99290309e+01, 8.36030627e-03, 1.01083444e+01,
                                   2.26556341e-02, 2.92301528e+03, 2.02545392e+02,
                                   9.35144361e+04, 4.30168535e+04, -2.04359112e+04],
                                  [7.99345918e+01, 7.82554947e-03, 1.01156067e+01,
                                   2.27701567e-02, 2.90760700e+03, 2.18884079e+02,
                                   9.17515503e+04, 5.35097619e+04, -2.05063843e+04],
                                  [7.99291265e+01, 8.20769719e-03, 1.01098599e+01,
                                   2.25719736e-02, 2.92108333e+03, 2.04567829e+02,
                                   9.30371060e+04, 4.71959588e+04, -2.04311251e+04],
                                  [7.99309311e+01, 8.16057122e-03, 1.01055938e+01,
                                   2.43439217e-02, 2.91825246e+03, 2.07749586e+02,
                                   9.64448932e+04, 1.26819672e+04, -2.07788349e+04],
                                  [7.99335199e+01, 8.52227957e-03, 1.01040059e+01,
                                   2.31349243e-02, 2.91924576e+03, 2.06796196e+02,
                                   9.39697966e+04, 3.49529061e+04, -2.04805003e+04],
                                  [7.99344734e+01, 8.48557085e-03, 1.01053300e+01,
                                   2.29537913e-02, 2.91715878e+03, 2.09002328e+02,
                                   9.33281586e+04, 4.01340309e+04, -2.04762004e+04]],

                                 [[7.99316033e+01, 8.58892543e-03, 1.01062145e+01,
                                   2.20683005e-02, 2.92263062e+03, 2.03079457e+02,
                                   9.21236995e+04, 5.46792478e+04, -2.04579997e+04],
                                  [7.99348754e+01, 8.00058291e-03, 1.01135261e+01,
                                   2.21828996e-02, 2.90956385e+03, 2.16879353e+02,
                                   9.06940459e+04, 6.40115169e+04, -2.04686810e+04],
                                  [7.99289507e+01, 8.35318087e-03, 1.01084108e+01,
                                   2.26739396e-02, 2.92302866e+03, 2.02527228e+02,
                                   9.35586895e+04, 4.26459089e+04, -2.04362580e+04],
                                  [7.99345918e+01, 7.82554947e-03, 1.01156067e+01,
                                   2.27701567e-02, 2.90760700e+03, 2.18884079e+02,
                                   9.17515503e+04, 5.35097619e+04, -2.05063843e+04],
                                  [7.99291265e+01, 8.20769719e-03, 1.01098599e+01,
                                   2.25719736e-02, 2.92108333e+03, 2.04567829e+02,
                                   9.30371060e+04, 4.71959588e+04, -2.04311251e+04],
                                  [7.99309311e+01, 8.16057122e-03, 1.01055938e+01,
                                   2.43439217e-02, 2.91825246e+03, 2.07749586e+02,
                                   9.64448932e+04, 1.26819672e+04, -2.07788349e+04],
                                  [7.99344188e+01, 8.64788798e-03, 1.01034544e+01,
                                   2.27150827e-02, 2.91952755e+03, 2.06533254e+02,
                                   9.31265880e+04, 4.25814894e+04, -2.04415480e+04],
                                  [7.99344734e+01, 8.48557085e-03, 1.01053300e+01,
                                   2.29537913e-02, 2.91715878e+03, 2.09002328e+02,
                                   9.33281586e+04, 4.01340309e+04, -2.04762004e+04]],

                                 [[7.99316033e+01, 8.58892543e-03, 1.01062145e+01,
                                   2.20683005e-02, 2.92263062e+03, 2.03079457e+02,
                                   9.21236995e+04, 5.46792478e+04, -2.04579997e+04],
                                  [7.99348754e+01, 8.00058291e-03, 1.01135261e+01,
                                   2.21828996e-02, 2.90956385e+03, 2.16879353e+02,
                                   9.06940459e+04, 6.40115169e+04, -2.04686810e+04],
                                  [7.99289507e+01, 8.35318087e-03, 1.01084108e+01,
                                   2.26739396e-02, 2.92302866e+03, 2.02527228e+02,
                                   9.35586895e+04, 4.26459089e+04, -2.04362580e+04],
                                  [7.99345918e+01, 7.82554947e-03, 1.01156067e+01,
                                   2.27701567e-02, 2.90760700e+03, 2.18884079e+02,
                                   9.17515503e+04, 5.35097619e+04, -2.05063843e+04],
                                  [7.99291898e+01, 8.20604559e-03, 1.01097104e+01,
                                   2.26340742e-02, 2.92099328e+03, 2.04669468e+02,
                                   9.31634628e+04, 4.59191418e+04, -2.04280112e+04],
                                  [7.99309311e+01, 8.16057122e-03, 1.01055938e+01,
                                   2.43439217e-02, 2.91825246e+03, 2.07749586e+02,
                                   9.64448932e+04, 1.26819672e+04, -2.07788349e+04],
                                  [7.99344089e+01, 8.69518650e-03, 1.01027555e+01,
                                   2.27119150e-02, 2.92022857e+03, 2.05806450e+02,
                                   9.32102106e+04, 4.19135473e+04, -2.04396332e+04],
                                  [7.99344962e+01, 8.48766653e-03, 1.01053283e+01,
                                   2.29448274e-02, 2.91715004e+03, 2.09012224e+02,
                                   9.33084988e+04, 4.03079998e+04, -2.04753505e+04]],

                                 [[7.99323332e+01, 8.61657145e-03, 1.01053146e+01,
                                   2.22357502e-02, 2.92200947e+03, 2.03784925e+02,
                                   9.24087417e+04, 5.13324879e+04, -2.04325453e+04],
                                  [7.99348754e+01, 8.00058291e-03, 1.01135261e+01,
                                   2.21828996e-02, 2.90956385e+03, 2.16879353e+02,
                                   9.06940459e+04, 6.40115169e+04, -2.04686810e+04],
                                  [7.99315178e+01, 8.11307054e-03, 1.01116855e+01,
                                   2.27177253e-02, 2.91613455e+03, 2.09836465e+02,
                                   9.27367178e+04, 4.75638055e+04, -2.04216750e+04],
                                  [7.99345918e+01, 7.82554947e-03, 1.01156067e+01,
                                   2.27701567e-02, 2.90760700e+03, 2.18884079e+02,
                                   9.17515503e+04, 5.35097619e+04, -2.05063843e+04],
                                  [7.99292032e+01, 8.20551134e-03, 1.01097217e+01,
                                   2.26345548e-02, 2.92096561e+03, 2.04698884e+02,
                                   9.31610117e+04, 4.59285281e+04, -2.04278187e+04],
                                  [7.99309586e+01, 8.16479657e-03, 1.01055714e+01,
                                   2.43310231e-02, 2.91826813e+03, 2.07734173e+02,
                                   9.64177754e+04, 1.29258006e+04, -2.07745043e+04],
                                  [7.99344089e+01, 8.69518650e-03, 1.01027555e+01,
                                   2.27119150e-02, 2.92022857e+03, 2.05806450e+02,
                                   9.32102106e+04, 4.19135473e+04, -2.04396332e+04],
                                  [7.99344962e+01, 8.48766653e-03, 1.01053283e+01,
                                   2.29448274e-02, 2.91715004e+03, 2.09012224e+02,
                                   9.33084988e+04, 4.03079998e+04, -2.04753505e+04]],

                                 [[7.99332613e+01, 8.65172140e-03, 1.01041704e+01,
                                   2.24486505e-02, 2.92117458e+03, 2.04730615e+02,
                                   9.27708572e+04, 4.70849511e+04, -2.04213216e+04],
                                  [7.99348754e+01, 8.00058291e-03, 1.01135261e+01,
                                   2.21828996e-02, 2.90956385e+03, 2.16879353e+02,
                                   9.06940459e+04, 6.40115169e+04, -2.04686810e+04],
                                  [7.99305118e+01, 8.20716901e-03, 1.01104021e+01,
                                   2.27005658e-02, 2.91885390e+03, 2.06952906e+02,
                                   9.30572306e+04, 4.56464333e+04, -2.04183565e+04],
                                  [7.99341984e+01, 7.86234467e-03, 1.01151049e+01,
                                   2.27634468e-02, 2.90872323e+03, 2.17699221e+02,
                                   9.18804288e+04, 5.27208212e+04, -2.04887631e+04],
                                  [7.99313450e+01, 8.12812854e-03, 1.01111583e+01,
                                   2.24640058e-02, 2.91680260e+03, 2.09144431e+02,
                                   9.22246017e+04, 5.27778928e+04, -2.04186836e+04],
                                  [7.99309586e+01, 8.16479657e-03, 1.01055714e+01,
                                   2.43310231e-02, 2.91826813e+03, 2.07734173e+02,
                                   9.64177754e+04, 1.29258006e+04, -2.07745043e+04],
                                  [7.99344089e+01, 8.69518650e-03, 1.01027555e+01,
                                   2.27119150e-02, 2.92022857e+03, 2.05806450e+02,
                                   9.32102106e+04, 4.19135473e+04, -2.04396332e+04],
                                  [7.99344962e+01, 8.48766653e-03, 1.01053283e+01,
                                   2.29448274e-02, 2.91715004e+03, 2.09012224e+02,
                                   9.33084988e+04, 4.03079998e+04, -2.04753505e+04]],

                                 [[7.99332613e+01, 8.65172140e-03, 1.01041704e+01,
                                   2.24486505e-02, 2.92117458e+03, 2.04730615e+02,
                                   9.27708572e+04, 4.70849511e+04, -2.04213216e+04],
                                  [7.99347258e+01, 8.14778774e-03, 1.01112378e+01,
                                   2.22977209e-02, 2.91190694e+03, 2.14442736e+02,
                                   9.12382973e+04, 5.92554045e+04, -2.04548938e+04],
                                  [7.99305118e+01, 8.20716901e-03, 1.01104021e+01,
                                   2.27005658e-02, 2.91885390e+03, 2.06952906e+02,
                                   9.30572306e+04, 4.56464333e+04, -2.04183565e+04],
                                  [7.99341997e+01, 7.98034340e-03, 1.01133551e+01,
                                   2.27533857e-02, 2.91036424e+03, 2.16002085e+02,
                                   9.20596497e+04, 5.12971233e+04, -2.04778936e+04],
                                  [7.99310955e+01, 8.15180037e-03, 1.01109318e+01,
                                   2.25348532e-02, 2.91742259e+03, 2.08481976e+02,
                                   9.24740698e+04, 5.06397500e+04, -2.04173328e+04],
                                  [7.99309586e+01, 8.16479657e-03, 1.01055714e+01,
                                   2.43310231e-02, 2.91826813e+03, 2.07734173e+02,
                                   9.64177754e+04, 1.29258006e+04, -2.07745043e+04],
                                  [7.99342074e+01, 8.65790083e-03, 1.01033080e+01,
                                   2.26956142e-02, 2.91999601e+03, 2.06033764e+02,
                                   9.31441879e+04, 4.26388451e+04, -2.04366187e+04],
                                  [7.99344962e+01, 8.48766653e-03, 1.01053283e+01,
                                   2.29448274e-02, 2.91715004e+03, 2.09012224e+02,
                                   9.33084988e+04, 4.03079998e+04, -2.04753505e+04]],

                                 [[7.99332613e+01, 8.65172140e-03, 1.01041704e+01,
                                   2.24486505e-02, 2.92117458e+03, 2.04730615e+02,
                                   9.27708572e+04, 4.70849511e+04, -2.04213216e+04],
                                  [7.99347258e+01, 8.14778774e-03, 1.01112378e+01,
                                   2.22977209e-02, 2.91190694e+03, 2.14442736e+02,
                                   9.12382973e+04, 5.92554045e+04, -2.04548938e+04],
                                  [7.99308445e+01, 8.18950154e-03, 1.01106330e+01,
                                   2.26988489e-02, 2.91814469e+03, 2.07709766e+02,
                                   9.29578858e+04, 4.62522532e+04, -2.04180149e+04],
                                  [7.99342832e+01, 8.00690492e-03, 1.01130193e+01,
                                   2.26811041e-02, 2.91061219e+03, 2.15751198e+02,
                                   9.19315166e+04, 5.25395220e+04, -2.04708999e+04],
                                  [7.99310955e+01, 8.15180037e-03, 1.01109318e+01,
                                   2.25348532e-02, 2.91742259e+03, 2.08481976e+02,
                                   9.24740698e+04, 5.06397500e+04, -2.04173328e+04],
                                  [7.99309586e+01, 8.16479657e-03, 1.01055714e+01,
                                   2.43310231e-02, 2.91826813e+03, 2.07734173e+02,
                                   9.64177754e+04, 1.29258006e+04, -2.07745043e+04],
                                  [7.99342074e+01, 8.65790083e-03, 1.01033080e+01,
                                   2.26956142e-02, 2.91999601e+03, 2.06033764e+02,
                                   9.31441879e+04, 4.26388451e+04, -2.04366187e+04],
                                  [7.99339916e+01, 8.55470116e-03, 1.01048551e+01,
                                   2.27420840e-02, 2.91879875e+03, 2.07258107e+02,
                                   9.30959470e+04, 4.30115295e+04, -2.04399044e+04]],

                                 [[7.99332613e+01, 8.65172140e-03, 1.01041704e+01,
                                   2.24486505e-02, 2.92117458e+03, 2.04730615e+02,
                                   9.27708572e+04, 4.70849511e+04, -2.04213216e+04],
                                  [7.99347258e+01, 8.14778774e-03, 1.01112378e+01,
                                   2.22977209e-02, 2.91190694e+03, 2.14442736e+02,
                                   9.12382973e+04, 5.92554045e+04, -2.04548938e+04],
                                  [7.99308445e+01, 8.18950154e-03, 1.01106330e+01,
                                   2.26988489e-02, 2.91814469e+03, 2.07709766e+02,
                                   9.29578858e+04, 4.62522532e+04, -2.04180149e+04],
                                  [7.99342832e+01, 8.00690492e-03, 1.01130193e+01,
                                   2.26811041e-02, 2.91061219e+03, 2.15751198e+02,
                                   9.19315166e+04, 5.25395220e+04, -2.04708999e+04],
                                  [7.99315707e+01, 8.26149028e-03, 1.01094483e+01,
                                   2.25159390e-02, 2.91822112e+03, 2.07685471e+02,
                                   9.25327762e+04, 4.99180178e+04, -2.04176037e+04],
                                  [7.99321101e+01, 8.33957421e-03, 1.01047691e+01,
                                   2.37513629e-02, 2.91882723e+03, 2.07189206e+02,
                                   9.52091740e+04, 2.38537137e+04, -2.06049899e+04],
                                  [7.99342074e+01, 8.65790083e-03, 1.01033080e+01,
                                   2.26956142e-02, 2.91999601e+03, 2.06033764e+02,
                                   9.31441879e+04, 4.26388451e+04, -2.04366187e+04],
                                  [7.99340380e+01, 8.57689478e-03, 1.01045224e+01,
                                   2.27320904e-02, 2.91905507e+03, 2.06996044e+02,
                                   9.31060577e+04, 4.29337649e+04, -2.04391352e+04]],

                                 [[7.99332613e+01, 8.65172140e-03, 1.01041704e+01,
                                   2.24486505e-02, 2.92117458e+03, 2.04730615e+02,
                                   9.27708572e+04, 4.70849511e+04, -2.04213216e+04],
                                  [7.99333819e+01, 8.16223130e-03, 1.01110284e+01,
                                   2.24366129e-02, 2.91412848e+03, 2.12044815e+02,
                                   9.18313975e+04, 5.47644826e+04, -2.04322697e+04],
                                  [7.99308445e+01, 8.18950154e-03, 1.01106330e+01,
                                   2.26988489e-02, 2.91814469e+03, 2.07709766e+02,
                                   9.29578858e+04, 4.62522532e+04, -2.04180149e+04],
                                  [7.99342832e+01, 8.00690492e-03, 1.01130193e+01,
                                   2.26811041e-02, 2.91061219e+03, 2.15751198e+02,
                                   9.19315166e+04, 5.25395220e+04, -2.04708999e+04],
                                  [7.99315707e+01, 8.26149028e-03, 1.01094483e+01,
                                   2.25159390e-02, 2.91822112e+03, 2.07685471e+02,
                                   9.25327762e+04, 4.99180178e+04, -2.04176037e+04],
                                  [7.99325973e+01, 8.47168213e-03, 1.01045157e+01,
                                   2.32000249e-02, 2.91992097e+03, 2.06040347e+02,
                                   9.42048250e+04, 3.34170461e+04, -2.04752432e+04],
                                  [7.99342074e+01, 8.65790083e-03, 1.01033080e+01,
                                   2.26956142e-02, 2.91999601e+03, 2.06033764e+02,
                                   9.31441879e+04, 4.26388451e+04, -2.04366187e+04],
                                  [7.99328393e+01, 8.42366212e-03, 1.01069155e+01,
                                   2.26270778e-02, 2.91864500e+03, 2.07336274e+02,
                                   9.28277611e+04, 4.63247964e+04, -2.04230714e+04]]]))
